#include <wxnc/color.hpp>
#include <wx/defs.h>
#include <cmath>
#include <ncurses.h>
#include "internal.h"

struct rgb {
	unsigned char r, g, b;
};

const wxncColor wxncBLACK(0);
const wxncColor wxncRED(1);
const wxncColor wxncGREEN(2);
const wxncColor wxncBROWN(3);
const wxncColor wxncBLUE(4);
const wxncColor wxncMAGENTA(5);
const wxncColor wxncCYAN(6);
const wxncColor wxncGRAY(7);
const wxncColor wxncDARKGRAY(8);
const wxncColor wxncHRED(9);
const wxncColor wxncHGREEN(10);
const wxncColor wxncYELLOW(11);
const wxncColor wxncHBLUE(12);
const wxncColor wxncHMAGENTA(13);
const wxncColor wxncHCYAN(14);
const wxncColor wxncWHITE(15);

/*
 * Some edge/text colors have reduced contrast for a given background -
 * try to mitigate by picking alternate colors.
 */
unsigned char wxncColor::b_coherence_map[][4] = {
	             /* edge+, edge-, sym, text */
	/* black         */ { 7, 8,  7,  7},
	/* red           */ {15, 8, 15,  7},
	/* green         */ {15, 8, 15,  0},
	/* brown         */ {15, 8, 15,  7},
	/* blue          */ {15, 0, 15,  7},
	/* magenta       */ {15, 8, 15, 15},
	/* cyan          */ {15, 8, 15,  0},
	/* gray          */ {15, 8, 15,  0},
};

static struct rgb vga_palette[] __attribute__((used)) = {
	/* black         */ {  0,   0,   0},
	/* red           */ {170,   0,   0},
	/* green         */ {  0, 170,   0},
	/* brown         */ { 85, 170,   0},
	/* blue          */ {  0,   0, 170},
	/* magenta       */ {170,   0, 170},
	/* cyan          */ {  0, 170, 170},
	/* gray          */ {170, 170, 170},
	/* darkgray      */ { 85,  85,  85},
	/* brightred     */ {255, 170, 170},
	/* brightgreen   */ {170, 255, 170},
	/* yellow        */ {255, 255, 170},
	/* brightblue    */ {170, 170, 255},
	/* brightmagenta */ {255, 170, 255},
	/* brightcyan    */ {170, 255, 255},
	/* white         */ {255, 255, 255},
};

static struct rgb vga2_palette[] __attribute__((used)) = {
	/* black         */ {  0,   0,   0},
	/* red           */ {170,   0,   0},
	/* green         */ {  0, 170,   0},
	/* brown         */ { 85, 170,   0},
	/* blue          */ {  0,   0, 170},
	/* magenta       */ {170,   0, 170},
	/* cyan          */ {  0, 170, 170},
	/* gray          */ {170, 170, 170},
	/* darkgray      */ { 85,  85,  85},
	/* brightred     */ {255,   0,   0},
	/* brightgreen   */ {  0, 255,   0},
	/* yellow        */ {255, 255,   0},
	/* brightblue    */ {  0,   0, 255},
	/* brightmagenta */ {255,   0, 255},
	/* brightcyan    */ {  0, 255, 255},
	/* white         */ {255, 255, 255},
};

static struct rgb xterm_palette[] __attribute__((used)) = {
	/* black         */ {  0,   0,   0},
	/* red           */ {170,   0,   0},
	/* green         */ {  0, 170,   0},
	/* brown         */ {170, 170,   0},
	/* blue          */ {  0,   0, 170},
	/* magenta       */ {170,   0, 170},
	/* cyan          */ {  0, 170, 170},
	/* gray          */ {170, 170, 170},
	/* darkgray      */ { 85,  85,  85},
	/* brightred     */ {255,   0,   0},
	/* brightgreen   */ {  0, 255,   0},
	/* yellow        */ {255, 255,   0},
	/* brightblue    */ {  0,   0, 255},
	/* brightmagenta */ {255,   0, 255},
	/* brightcyan    */ {  0, 255, 255},
	/* white         */ {255, 255, 255},
};

static inline int ipow2(int x)
{
	return x * x;
}

static inline double dist_std(const struct rgb *t,
    unsigned char sr, unsigned char sg, unsigned char sb)
{
	return sqrt(ipow2(sr - t->r) + ipow2(sg - t->g) +
	       ipow2(sb - t->b));
}

wxncColor::wxncColor(void)
{
	m_bgcand = m_fgcand = 7;
	m_invert = false;
}

wxncColor::wxncColor(unsigned char x)
{
	m_bgcand = x & ~0x08;
	m_fgcand = x;
	m_invert = false;
}

wxncColor::wxncColor(unsigned char r, unsigned char g, unsigned char b)
{
	double delta[16], min_bgcand = 1E32, min_fgcand = 1E32;

	for (unsigned int i = 0; i < WXSIZEOF(vga2_palette); ++i) {
		delta[i] = dist_std(&vga2_palette[i], r, g, b);
		if (i < 8 && delta[i] < min_bgcand) {
			min_bgcand = delta[i];
			m_bgcand   = i;
		}
		if (delta[i] < min_fgcand) {
			min_fgcand = delta[i];
			m_fgcand   = i;
		}
	}
}

wxncColor::~wxncColor(void)
{
}

void wxncColor::SetInvert(bool x)
{
	m_invert = x;
}

wxncRenderColor wxncColor::GetColor(const wxncColor &f, const wxncColor &b)
{
	wxncRenderColor c;

	if (!has_colors())
		return ((f.m_fgcand & 0x8) ? A_BOLD : 0) |
		       (b.m_invert ? A_REVERSE : 0);

	c  = 8 * (b.m_bgcand & ~0x8);
	c |= f.m_fgcand & ~0x8;
	c  = COLOR_PAIR(c);
	c |= (f.m_fgcand & 0x8) ? A_BOLD : 0;
	return c;
}

bool wxncColor::fgbg_compare(const wxncColor &fg, const wxncColor &bg)
{
	return fg.m_fgcand == bg.m_bgcand;
}

unsigned char wxncColor::nc_index(void) const
{
	return m_fgcand;
}

/* simplistic */
/*
unsigned char wxncColor::GetColor2(void) const
{
	return (color_idx & 0x07) | (color_bold ? 0x08 : 0x00) |
	       ((color_idx & 0x38) << 1);
}
*/