#include <list>
#include <wxnc/app.hpp>
#include <wxnc/color.hpp>
#include <wxnc/dialog.hpp>
#include <wxnc/frame.hpp>
#include <wxnc/menu.hpp>
#include <wxnc/menubar.hpp>
#include <wxnc/panel.hpp>
#include <wxnc/statusbar.hpp>
#include <wxnc/window.hpp>
#include <ncurses.h>
#include "internal.h"

class wxncBackFrame : public wxncPanel {
    public:
	wxncBackFrame(void);
	void Update(void);
};

wxncBackFrame::wxncBackFrame(void) :
	wxncPanel(NULL)
{
	SetBackgroundColor(wxncBLUE);
}

void wxncBackFrame::Update(void)
{
	if (!has_colors())
		return;

	/*
	 * Checkerboard background pattern, to simulate a 50%-50% color mix.
	 * Been done commonly in DOS apps. It works well for Xterm,
	 * but not always for all VGA fonts due to the 8th column not
	 * being replicated.
	 */
#if 0
	attrset(wxncColor::GetColor(wxncBLUE, m_bgcolor));
	for (int y = m_pos.y; y < m_pos.y + m_size.y; ++y)
		for (int x = m_pos.x; x < m_size.x; ++x)
			mvaddch(y, x, ACS_CKBOARD);
#else
	/* use standard background */
	wxncWindow::Update();
#endif
}

wxncFrame::wxncFrame(wxncWindow *parent, wxWindowID id, const wxString &title,
    const wxPoint &pos, const wxSize &size, long style, const wxString &name) :
	wxncTopLevelWindow(parent, id, title, pos, size,
		style | wxBORDER_NONE, name),
	m_menubar(NULL),
	m_statusbar(NULL)
{
	if (m_pos == wxDefaultPosition)
		m_pos = wxPoint(0, 0);
	if (m_size == wxDefaultSize)
		m_size = wxSize(stdscr->_maxx + 1, stdscr->_maxy + 1);

	m_backframe = new wxncBackFrame();
	wxncGetApp().replace_frame(this);
}

wxncFrame::~wxncFrame(void)
{
	delete m_backframe;
	delete m_menubar;
	delete m_statusbar;
}

wxncStatusBar *wxncFrame::CreateStatusBar(int number, long style,
    wxWindowID id, const wxString &name)
{
	wxncStatusBar *sb = new wxncStatusBar(this, id, style, name);

	sb->SetFieldsCount(number);
	SetStatusBar(sb);
	return sb;
}

void wxncFrame::SetStatusText(const wxString &text, int col)
{
	m_statusbar->SetStatusText(text, col);
}

void wxncFrame::SetMenuBar(wxncMenuBar *mb)
{
	delete m_menubar;
	m_menubar = mb;
}

void wxncFrame::SetStatusBar(wxncStatusBar *sb)
{
	delete m_statusbar;
	m_statusbar = sb;
}

void wxncFrame::RecalcSizes(void)
{
	if (m_menubar != NULL)
		m_menubar->RecalcSizes();
	if (m_statusbar != NULL)
		m_statusbar->RecalcSizes();

	wxSize saved_size = m_minsize;
	wxncWindow::RecalcSizes();
	m_minsize = saved_size;
}

void wxncFrame::Layout(void)
{
	m_backframe->SetPosition(m_pos);
	m_backframe->SetSize(m_size);
	m_clientsize = m_size;

	if (m_menubar != NULL) {
		m_menubar->SetPosition(m_pos);
		m_menubar->SetSize(m_size.x, 1);
		m_menubar->Layout();
		m_backframe->SetPosition(m_pos.x, m_backframe->GetPosition().y + 1);
		m_backframe->SetSize(m_backframe->GetWidth(), m_backframe->GetHeight() - 1);
	}
	if (m_statusbar != NULL) {
		m_statusbar->SetPosition(m_pos.x, m_pos.y + m_size.y - 1);
		m_statusbar->SetSize(m_size.x, 1);
		m_statusbar->Layout();
		m_backframe->SetSize(m_backframe->GetWidth(), m_backframe->GetHeight() - 1);
	}

	m_clientsize = m_backframe->GetSize();
	m_clientpos  = m_backframe->GetPosition();

	for (std::list<wxncWindow *>::const_iterator
	     w = m_window_list.begin(); w != m_window_list.end(); ++w)
	{
		wxSize s = (*w)->GetMinSize();
		s.DecTo(m_clientsize);
		(*w)->SetSize(s);
		/* Children of wxncFrame will get centered on the frame */
		(*w)->SetPosition((m_clientsize.x - s.x) / 2, (m_clientsize.y - s.y) / 2);
		(*w)->Layout();
	}
}

void wxncFrame::Update(void)
{
	if (m_menubar != NULL)
		m_menubar->Update();
	if (m_statusbar != NULL)
		m_statusbar->Update();
	wxncWindow::Update();
}

void wxncFrame::render_background(int sx, int sy, int ex, int ey)
{
	m_backframe->Update();
}

wxncDialog::wxncDialog(wxncWindow *parent, wxWindowID id,
    const wxString &title, const wxPoint &pos, const wxSize &size,
    long style, const wxString &name) :
	wxncTopLevelWindow(parent, id, title, pos, size, style, name)
{
}

wxncDialog::~wxncDialog(void)
{
}

wxncPanel::wxncPanel(wxncWindow *parent, wxWindowID id, const wxPoint &pos,
    const wxSize &size, long style, const wxString &name) :
	wxncWindow(parent, id, pos, size, style | wxBORDER_NONE, name)
{
}

wxncPanel::~wxncPanel(void)
{
}
